// @flow
import * as React from 'react';
// $FlowFixMe
import { Text, Platform, StyleSheet } from 'react-native';

const styles = StyleSheet.create({
  text: {
    ...Platform.select({
      web: {
        WebkitFontSmoothing: 'antialiased',
        MozOsxFontSmoothing: 'grayscale',
        fontFamily: 'Roboto, sans-serif',
        fontSize: 13,
      },
      default: {
        fontFamily: 'Roboto',
        fontSize: 14,
      },
    }),
  },
  display4: {
    fontSize: 112,
    fontWeight: '300',
    letterSpacing: -4.48,
    lineHeight: 128,
    marginBottom: 64,
  },
  display3: {
    fontSize: 56,
    letterSpacing: -1.12,
    lineHeight: 72,
    marginBottom: 64,
  },
  display2: {
    fontSize: 45,
    lineHeight: 48,
    marginBottom: 64,
  },
  display1: {
    fontSize: 34,
    lineHeight: 40,
    marginBottom: 64,
  },
  headline: {
    fontSize: 24,
    lineHeight: 32,
    marginBottom: 16,
  },
  title: {
    fontSize: 20,
    fontWeight: '500',
    letterSpacing: 0.4,
    lineHeight: 24,
    marginBottom: 16,
  },
  subheading: {
    ...Platform.select({
      web: {
        fontSize: 15,
      },
      default: {
        fontSize: 16,
      },
    }),
    letterSpacing: 0.64,
    lineHeight: 24,
    marginBottom: 16,
  },
  body2: {
    fontWeight: '500',
    letterSpacing: 0.56,
    lineHeight: 24,
    marginBottom: 12,
  },
  body1: {
    letterSpacing: 0.56,
    lineHeight: 20,
    marginBottom: 12,
  },
  caption: {
    fontSize: 12,
    letterSpacing: 0.96,
    lineHeight: 16,
  },
  button: {
    fontSize: 14,
    fontWeight: '500',
    letterSpacing: 0.56,
    lineHeight: 16,
    textAlign: 'center',
  },
});

type Props = {
  color: string,
  text: string,
  type: 'display4' | 'display3' | 'display2' | 'display1' | 'headline' | 'title' | 'subheading' | 'body1' | 'body2' | 'caption' | 'button',
}

class Typography extends React.Component<Props> {
  static defaultProps = {
    color: '#000',
    type: 'body1',
    text: '',
  }

  setNativeProps = (nativeProps: Object) => {
    if (this.elem) {
      this.elem.setNativeProps(nativeProps);
    }
  }

  elem: ?Text

  render() {
    const { color, text, type } = this.props;
    let role;
    const typographyStyles = StyleSheet.flatten([
      styles.text,
      styles[type],
      color && { color },
    ]);

    switch (type) {
      case 'display1':
      case 'display2':
      case 'display3':
      case 'display4':
      case 'headline':
      case 'title':
      case 'subheading':
        role = 'heading';
        break;
      default:
        role = null;
    }
    const label = type !== 'button' ? text : text.toUpperCase();

    return (
      <Text
        style={typographyStyles}
        accessibilityRole={role}
        ref={(elem) => { this.elem = elem; }}
      >
        {label}
      </Text>
    );
  }
}

export default Typography;
